--[[
	Name: LibLocalizedEJBosses-1.0
	Revision: $Revision: 2 $
	Author: Caleb (calebzor@gmail.com)
	SVN:  svn://svn.wowace.com/wow/LibLocalizedEJBosses-1-0/mainline/trunk
	Description:  A library to get localized boss names from the Encounter Journal
	Dependencies: LibStub
]]

local MAJOR_VERSION = "LibLocalizedEJBosses-1.0"
local MINOR_VERSION = tonumber(("$Revision: 2 $"):match("%d+"))
local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

local bosses = {}
local descriptions = {}
local icons = {}
local initialized = nil

lejb = lib
-- this is used for the saved variable so a script can generate the bottom part of this file
local function savedVariableGenerationForScript()
	LEJBbossList = bosses
end

local function populateBossList(isRaid)
	local i = 1
	local instanceID = EJ_GetInstanceByIndex(i, isRaid)

	while instanceID do
		EJ_SelectInstance(instanceID)
		if tonumber((select(4, GetBuildInfo()))) < 40300 then
			EJ_SetDifficulty(true, false) -- heroic, 25 man
		else
			if isRaid then
				EJ_SetDifficulty(4) -- heroic, 25 man
			else
				EJ_SetDifficulty(2) -- heroic, 5 man
			end
		end
		local j = 1
		local name, desc, bossId = EJ_GetEncounterInfoByIndex(j)

		while name and j < 25 do
			bosses[bossId] = name
			descriptions[bossId] = desc
			local _, _, _, _, bossImage = EJ_GetCreatureInfo(j, bossId)
			icons[bossId] = bossImage or "Interface\\EncounterJournal\\UI-EJ-BOSS-Default"
			j = j + 1;
			name, desc, bossId = EJ_GetEncounterInfoByIndex(j);
		end
		i = i + 1;
		instanceID = EJ_GetInstanceByIndex(i, isRaid)
	end
	initialized = true
	--[===[@debug@
	-- XXX debug
	savedVariableGenerationForScript()
	--@end-debug@]===]
end

--- Get the localized name of a boss based on the Blizzard EJ bossNames
-- @see lib.enUSbossList for currently supported bosses
-- @return localized boss name, description, icon or nil if it does not exsits in the EJ
function lib:GetEncounterInfoByBossName(bossName)
	if not initialized then populateBossList(true) populateBossList(false) end
	local bossId = nil
	for k, v in pairs(bosses) do
		if v == bossName then
			bossId = k
		end
	end
	if not bossId then return nil end
	return bosses[bossId], descriptions[bossId], icons[bossId]
end

--- Get the localized name of a boss based on the Blizzard EJ bossIds
-- @see lib.enUSbossList for currently supported bosses
-- @return localized boss name, description, icon or nil if it does not exsits in the EJ
function lib:GetEncounterInfoByBossId(bossId)
	if not initialized then populateBossList(true) populateBossList(false) end
	return bosses[bossId], descriptions[bossId], icons[bossId]
end

-- DO NOT ADD ANYTHING BELOW THIS POINT
-- START enUS boss list for script --
lib.enUSbossList = {
	[122] = "Siamat",
	[124] = "Temple Guardian Anhuur",
	[125] = "Earthrager Ptah",
	[126] = "Anraphet",
	[127] = "Isiset, Construct of Magic",
	[128] = "Ammunae, Construct of Life",
	[130] = "Rajh, Construct of Sun",
	[132] = "Forgemaster Throngus",
	[134] = "Erudax, the Duke of Below",
	[140] = "Occu'thar",
	[290] = "Peroth'arn",
	[154] = "The Conclave of Wind",
	[156] = "Halfus Wyrmbreaker",
	[158] = "Ascendant Council",
	[318] = "Spine of Deathwing",
	[322] = "Arcurion",
	[168] = "Sinestra",
	[170] = "Magmaw",
	[172] = "Chimaeron",
	[174] = "Nefarian's End",
	[176] = "Bloodlord Mandokir",
	[178] = "Cache of Madness - Hazza'rah",
	[180] = "Cache of Madness - Wushoolay",
	[184] = "Zanzil",
	[186] = "Akil'zon",
	[188] = "Jan'alai",
	[190] = "Hex Lord Malacrass",
	[192] = "Beth'tilac",
	[194] = "Alysrazor",
	[196] = "Baleroc, the Gatekeeper",
	[198] = "Ragnaros",
	[283] = "Echo of Tyrande",
	[291] = "Queen Azshara",
	[311] = "Morchok",
	[323] = "Echo of Sylvanas",
	[331] = "Ultraxion",
	[339] = "Alizabal, Mistress of Hate",
	[185] = "Jin'do the Godbreaker",
	[181] = "High Priestess Kilnara",
	[179] = "Cache of Madness - Renataki",
	[177] = "Cache of Madness - Gri'lek",
	[112] = "Ozruk",
	[129] = "Setesh, Construct of Destruction",
	[131] = "General Umbriss",
	[133] = "Drahga Shadowburner",
	[191] = "Daakara",
	[189] = "Halazzi",
	[139] = "Argaloth",
	[342] = "Asira Dawnslayer",
	[340] = "Echo of Baine",
	[116] = "Asaad, Caliph of Zephyrs",
	[292] = "Mannoroth and Varo'then",
	[114] = "Grand Vizier Ertan",
	[104] = "Ozumat",
	[98] = "Commander Springvale",
	[155] = "Al'Akir",
	[157] = "Theralion and Valiona",
	[99] = "Lord Walden",
	[97] = "Baron Silverlaine",
	[324] = "Warlord Zon'ozz",
	[118] = "Lockmaw",
	[332] = "Warmaster Blackhorn",
	[169] = "Omnotron Defense System",
	[171] = "Atramedes",
	[173] = "Maloriak",
	[175] = "High Priest Venoxis",
	[89] = "Glubtok",
	[90] = "Helix Gearbreaker",
	[91] = "Foe Reaper 5000",
	[92] = "Admiral Ripsnarl",
	[93] = "\"Captain\" Cookie",
	[187] = "Nalorakk",
	[95] = "Vanessa VanCleef",
	[96] = "Baron Ashbury",
	[193] = "Lord Rhyolith",
	[195] = "Shannox",
	[197] = "Majordomo Staghelm",
	[100] = "Lord Godfrey",
	[101] = "Lady Naz'jar",
	[102] = "Commander Ulthok, the Festering Prince",
	[103] = "Mindbender Ghur'sha",
	[285] = "Echo of Jaina",
	[105] = "Rom'ogg Bonecrusher",
	[106] = "Corla, Herald of Twilight",
	[107] = "Karsh Steelbender",
	[108] = "Beauty",
	[109] = "Ascendant Lord Obsidius",
	[110] = "Corborus",
	[111] = "Slabhide",
	[317] = "Hagara the Stormbinder",
	[113] = "High Priestess Azil",
	[325] = "Yor'sahj the Unsleeping",
	[115] = "Altairus",
	[333] = "Madness of Deathwing",
	[117] = "General Husam",
	[341] = "Archbishop Benedictus",
	[119] = "High Prophet Barim",
	[167] = "Cho'gall",
	[289] = "Murozond",
}
-- END enUS boss list for script --

--- Bosses.
--@description The list of supported bosses: bossId, bossName
--@class table
--@name lib.enUSbossList
--@field 89 Glubtok
--@field 90 Helix Gearbreaker
--@field 91 Foe Reaper 5000
--@field 92 Admiral Ripsnarl
--@field 93 "Captain" Cookie
--@field 95 Vanessa VanCleef
--@field 96 Baron Ashbury
--@field 97 Baron Silverlaine
--@field 98 Commander Springvale
--@field 99 Lord Walden
--@field 100 Lord Godfrey
--@field 101 Lady Naz'jar
--@field 102 Commander Ulthok, the Festering Prince
--@field 103 Mindbender Ghur'sha
--@field 104 Ozumat
--@field 105 Rom'ogg Bonecrusher
--@field 106 Corla, Herald of Twilight
--@field 107 Karsh Steelbender
--@field 108 Beauty
--@field 109 Ascendant Lord Obsidius
--@field 110 Corborus
--@field 111 Slabhide
--@field 112 Ozruk
--@field 113 High Priestess Azil
--@field 114 Grand Vizier Ertan
--@field 115 Altairus
--@field 116 Asaad, Caliph of Zephyrs
--@field 117 General Husam
--@field 118 Lockmaw
--@field 119 High Prophet Barim
--@field 122 Siamat
--@field 124 Temple Guardian Anhuur
--@field 125 Earthrager Ptah
--@field 126 Anraphet
--@field 127 Isiset, Construct of Magic
--@field 128 Ammunae, Construct of Life
--@field 129 Setesh, Construct of Destruction
--@field 130 Rajh, Construct of Sun
--@field 131 General Umbriss
--@field 132 Forgemaster Throngus
--@field 133 Drahga Shadowburner
--@field 134 Erudax, the Duke of Below
--@field 139 Argaloth
--@field 140 Occu'thar
--@field 154 The Conclave of Wind
--@field 155 Al'Akir
--@field 156 Halfus Wyrmbreaker
--@field 157 Theralion and Valiona
--@field 158 Ascendant Council
--@field 167 Cho'gall
--@field 168 Sinestra
--@field 169 Omnotron Defense System
--@field 170 Magmaw
--@field 171 Atramedes
--@field 172 Chimaeron
--@field 173 Maloriak
--@field 174 Nefarian's End
--@field 175 High Priest Venoxis
--@field 176 Bloodlord Mandokir
--@field 177 Cache of Madness - Gri'lek
--@field 178 Cache of Madness - Hazza'rah
--@field 179 Cache of Madness - Renataki
--@field 180 Cache of Madness - Wushoolay
--@field 181 High Priestess Kilnara
--@field 184 Zanzil
--@field 185 Jin'do the Godbreaker
--@field 186 Akil'zon
--@field 187 Nalorakk
--@field 188 Jan'alai
--@field 189 Halazzi
--@field 190 Hex Lord Malacrass
--@field 191 Daakara
--@field 192 Beth'tilac
--@field 193 Lord Rhyolith
--@field 194 Alysrazor
--@field 195 Shannox
--@field 196 Baleroc, the Gatekeeper
--@field 197 Majordomo Staghelm
--@field 198 Ragnaros
--@field 283 Echo of Tyrande
--@field 285 Echo of Jaina
--@field 289 Murozond
--@field 290 Peroth'arn
--@field 291 Queen Azshara
--@field 292 Mannoroth and Varo'then
--@field 311 Morchok
--@field 317 Hagara the Stormbinder
--@field 318 Spine of Deathwing
--@field 322 Arcurion
--@field 323 Echo of Sylvanas
--@field 324 Warlord Zon'ozz
--@field 325 Yor'sahj the Unsleeping
--@field 331 Ultraxion
--@field 332 Warmaster Blackhorn
--@field 333 Madness of Deathwing
--@field 339 Alizabal, Mistress of Hate
--@field 340 Echo of Baine
--@field 341 Archbishop Benedictus
--@field 342 Asira Dawnslayer
